#include "rotarydial.h"

#include "app_timer.h"

#include "nrf_gpio.h"
#include "nrf_gpiote.h"
#include "app_gpiote.h"
#include "nrf_delay.h"
#include "bsp.h"

#define RD_PIN_A 12
#define RD_PIN_B 13

#define START_DEBOUNCE_A do {app_timer_start(m_rd_timer_a, APP_TIMER_TICKS(25, 0), NULL);} while(0);
#define START_DEBOUNCE_B do {app_timer_start(m_rd_timer_b, APP_TIMER_TICKS(10, 0), NULL);} while(0);



#define RD_STATE_IDLE 0
#define RD_STATE_ACTIVE 1

static volatile uint32_t m_state = RD_STATE_IDLE;

static volatile uint32_t m_b_state;
static volatile uint32_t m_b_count;

static bool m_debounce_a = false;
static bool m_debounce_b = false;
static app_timer_id_t m_rd_timer_a;
static app_timer_id_t m_rd_timer_b;


static void rd_timer_a_handler(void * p_context)
{
    (void)(p_context);
    m_debounce_a=false;
    LEDS_OFF(BSP_LED_1_MASK);
}

static void rd_timer_b_handler(void * p_context)
{
    (void)(p_context);
    m_debounce_b=false;
    //LEDS_OFF(BSP_LED_0_MASK);
}

void rd_a_handler(uint32_t event_pins_low_to_high,
                         uint32_t event_pins_high_to_low)
{
    nrf_delay_ms(15);
        uint32_t pinstate = nrf_gpio_pin_read(RD_PIN_A);
        if (!m_debounce_a)
        {
            m_debounce_a=true;
            LEDS_ON(BSP_LED_1_MASK);
            START_DEBOUNCE_A;
            if (pinstate)
            {
                m_state = RD_STATE_IDLE;
                LEDS_OFF(BSP_LED_3_MASK);
                rd_out(m_b_count);
            }
            else
            {
                m_state = RD_STATE_ACTIVE;
                m_b_count=0;
                LEDS_ON(BSP_LED_3_MASK);
            }
        }
}
    
void rd_b_handler(uint32_t event_pins_low_to_high,
                         uint32_t event_pins_high_to_low)
    {
        
        uint32_t pinstate = nrf_gpio_pin_read(RD_PIN_B);
        if (!m_debounce_b)
        {
            m_debounce_b=true;
            //LEDS_ON(BSP_LED_0_MASK);
            START_DEBOUNCE_B;
            if (!pinstate)
            {
                m_b_state = RD_STATE_IDLE;
                LEDS_OFF(BSP_LED_2_MASK);
            }
            else
            {
                m_b_state = RD_STATE_ACTIVE;
                ++m_b_count;
                LEDS_ON(BSP_LED_2_MASK);
            }
        }
    }

void rd_init()
{
    uint32_t err_code;
    
    //GPIO and GPIOTE init:
    nrf_gpio_cfg_input(RD_PIN_A, NRF_GPIO_PIN_PULLUP);
    nrf_gpio_cfg_input(13, NRF_GPIO_PIN_PULLUP);
    nrf_gpio_cfg_output(14);
    nrf_gpio_pin_clear(14);
    
    app_gpiote_user_id_t uid;
    err_code = app_gpiote_user_register(&uid,
                                        1<<RD_PIN_A,
                                        1<<RD_PIN_A,
                                        rd_a_handler);
    app_gpiote_user_enable(uid); 
    err_code = app_gpiote_user_register(&uid,
                                        1<<RD_PIN_B,
                                        1<<RD_PIN_B,
                                        rd_b_handler);
    app_gpiote_user_enable(uid);       

    // Debounce timers init:
    err_code = app_timer_create(&m_rd_timer_a,
                                APP_TIMER_MODE_SINGLE_SHOT,
                                rd_timer_a_handler);
    APP_ERROR_CHECK(err_code);
    err_code = app_timer_create(&m_rd_timer_b,
                                APP_TIMER_MODE_SINGLE_SHOT,
                                rd_timer_b_handler);
    APP_ERROR_CHECK(err_code);
}
